#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
# Clay.manage

Command line script.

"""
import io
import os
import sys

from shake import manager, json, voodoo

from .core import Clay


WELCOME = """
\033[1m Clay \033[0m - A rapid prototyping tool by Lucuma labs"""

SKELETON = os.path.join(os.path.dirname(os.path.realpath(__file__)),
    'skeleton')

SKELETON_HELP = """
    Done!
    Now go to %(new_app_path)s, and run `clay run` to start the server.
    Don't forget to read the README.md
    """

JSON_COMMENTS = ('/', '#')


@manager.command
def new(new_app_path):
    """DIR_PATH

    Creates a new project
    """
    new_app_path = new_app_path.rstrip(os.path.sep)
    voodoo.reanimate_skeleton(SKELETON, new_app_path)

    print SKELETON_HELP % {'new_app_path': new_app_path}


@manager.command
def build(cwd=None):
    """
    Generates a static version of the site
    """
    proto = get_current(cwd)
    proto.build()


@manager.command
def make(cwd=None):
    """
    An alias for 'clay build' (backwards compatibility)
    """
    return build(cwd)


@manager.command
def run(cwd=None):
    """
    Run the development server
    """
    proto = get_current(cwd)
    proto.run()


def get_current(cwd=None):
    cwd = '.' if cwd is None else cwd
    cwd = os.path.abspath(cwd)
    # print cwd
    settings = get_settings(cwd)
    proto = Clay(cwd, settings)
    return proto


def get_settings(cwd, filename='settings.json'):
    settings = {}
    settings_filepath = os.path.join(cwd, filename)
    if os.path.isfile(settings_filepath):
        with io.open(settings_filepath) as f:
            settings_json = ''.join([l for l in f.readlines()
                if not l.strip().startswith(JSON_COMMENTS)])
        try:
            settings = json.loads(settings_json)
        except (ValueError, SyntaxError):
            settings = {}
    return settings


def main():
    manager.run(prefix=WELCOME)


if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
"""
    # Clay.processors.markdown_jinja

    A markdown pre-compiler for jinja templates.
    http://www.freewisdom.org/projects/python-markdown/

"""
import os
from string import Template

from jinja2.ext import Extension
import markdown


extensions_in = ['.md', '.markdown']

WRAPPER = Template("""{% extends "$base" %}

{% block title %}$title{% endblock %}

{% block content %}
$_html
{% endblock %}
""")


def get_extension(settings):

    md = markdown.Markdown(
        extensions=settings.markdown_extensions,
        safe_mode=settings.markdown_safe_mode,
        output_format=settings.markdown_output_format
    )

    class MarkdownExtension(Extension):
        
        def preprocess(self, source, name, filename=None):
            if not os.path.splitext(name)[1] in extensions_in:
                return source
            
            html = md.convert(source)
            meta = md.Meta
            data = {}
            data['title'] = meta.get('title', [u''])[0].strip()
            data['base'] = meta.get('base', [u'base.html'])[0].strip()
            data['_html'] = html

            return WRAPPER.substitute(data)
    
    return MarkdownExtension




# -*- coding: utf-8 -*-
"""
    # Clay.processors.less

    http://lesscss.org/

"""
import io
import subprocess


COMMAND = 'lessc'

try:
    subprocess.check_output([COMMAND, '--version'])
    enabled = True
except OSError:
    enabled = False

extensions_in = ['.less']
extension_out = '.css'


def render(path, settings):
    content = subprocess.check_output([COMMAND, path])
    return content, extension_out




# -*- coding: utf-8 -*-
"""
    # Clay.config
"""

VIEWS_DIR = 'views'
BUILD_DIR = 'build'
VIEWS_INDEX = '_index.html'

IGNORE = ('.', '_')

default_settings = {
    'views_list_ignore': [],

    # CoffeeScript settings
    'coffee_bare': True,
    
    # Markdown settings 
    'markdown_extensions': [
        # Abbreviations
        'abbr',
        # Definition lists
        'def_list',
        # Footnotes
        'footnotes',
        # Fenced code blocks
        'fenced_code',
        # HeaderId
        'headerid',
        # Tables
        'tables',
        # Code highlighting using using Pygments
        'codehilite',
        # Treat newlines as hard breaks (like StackOverflow and GitHub).
        'nl2br',
        # Table of Contents
        'toc',
        # Meta-Data
        'meta',
    ], 
    'markdown_safe_mode': False,
    'markdown_output_format': 'xhtml1',

    'host': '0.0.0.0',
    'port': 5000,
}





# -*- coding: utf-8 -*-
"""
# Clay.utils

"""
from datetime import datetime
import errno
import io
import math
import os
import re


rx_abs_url = re.compile(r' (src|href)=[\'"]\/')


def walk_dir(path, callback, ignore=None):
    ignore = ignore or ()
    for folder, subs, files in os.walk(path):
        ffolder = os.path.relpath(folder, path)
        for filename in files:
            if filename.startswith(ignore):
                continue
            relpath = os.path.join(ffolder, filename) \
                .lstrip('.').lstrip('/')
            callback(relpath)


def make_dirs(*lpath):
    path = os.path.join(*lpath)
    try:
        os.makedirs(os.path.dirname(path))
    except (OSError), e:
        if e.errno != errno.EEXIST:
            raise
    return path


def get_source(filepath):
    source = ''
    with io.open(filepath) as f:
        source = f.read()
    return source


def make_file(filepath, content):
    if not isinstance(content, unicode):
        content = unicode(content, 'utf-8')
    with io.open(filepath, 'w+t') as f:
        f.write(content)


def absolute_to_relative(content, relpath):
    depth = relpath.count(os.path.sep)
    repl = '../' * depth
    rx_rel_url = r' \1="%s' % repl
    content = re.sub(rx_abs_url, rx_rel_url, content)
    return content


def get_processed_regex(processed_files):
    rx_processed = [[
        re.compile(r' (src|href)=[\'"](.*)%s[\'"]' % old),
        r' \1="\2%s"' % new
    ] for old, new in processed_files]
    return rx_processed


def replace_processed_names(content, rx_processed):
    for rxold, rxnew in rx_processed:
        content = re.sub(rxold, rxnew, content)
    return content


def get_file_mdate(filepath):
    st = os.stat(filepath)
    mdate = datetime.utcfromtimestamp(st.st_mtime)
    mdate -=  datetime.utcnow() - datetime.now()
    return mdate




# -*- coding: utf-8 -*-
"""
# Clay

A rapid prototyping tool


Copyright © 2011 by Lúcuma labs (http://lucumalabs.com).
Coded by Juan-Pablo Scaletti <juanpablo@lucumalabs.com>.
MIT License. (http://www.opensource.org/licenses/mit-license.php).

"""
import mimetypes
import os
import shutil

from shake import (Shake, Settings, Rule, NotFound,
    TemplateNotFound, Response, send_file)

from . import utils
from .config import *
from .render import Render


class Clay(object):

    def __init__(self, base_dir, settings=None):
        if os.path.isfile(base_dir):
            base_dir = os.path.abspath(os.path.dirname(base_dir))
        self.base_dir = base_dir
        self.settings = settings
        self.views_dir = utils.make_dirs(base_dir, VIEWS_DIR)
        self.build_dir = os.path.join(base_dir, BUILD_DIR)

        self.settings = Settings(default_settings, settings,
            case_insensitive=True)
        
        self.app = Shake(self._get_urls())
        self.render = Render(self.views_dir, self.settings)
    
    def run(self, host=None, port=None):
        host = host or self.settings.host
        port = port or self.settings.port
        return self.app.run(host, port)
    
    def build(self):
        """Generates a static version of the site.
        """
        print '\nGenerating views...\n', '-' * 20
        self.build_views()
        print '\nDone!\n'
    
    def test_client(self):
        return self.app.test_client()
    
    def _get_urls(self):
        return [
            Rule('/', self.render_view),
            Rule('/<path:path>', self.render_view),
        ]
    
    def _normalize_path(self, path):
        if '..' in path:
            return self.not_found()
        path = path.strip('/')
        is_dir = os.path.isdir(os.path.join(self.views_dir, path))
        if is_dir:
            path += '/'
        if not path or is_dir:
            path += 'index.html'
        return path
    
    def render_view(self, request, path=''):
        """Default controller.
        Render the template at `path` guessing it's mimetype.
        """
        path = self._normalize_path(path)
        try:
            content, ext = self.render(path)
            mimetype = mimetypes.guess_type('a' + ext)[0] or 'text/plain'
        except TemplateNotFound:
            return self.not_found()
        if content:
            return Response(content, mimetype=mimetype)
        
        fullpath = os.path.join(self.views_dir, path)
        if not os.path.isfile(fullpath):
            return self.not_found() 
        return send_file(request, fullpath)
    
    def build_views(self):
        processed = []
        views = []

        def callback(relpath_in):
            print relpath_in
            fn, old_ext = os.path.splitext(relpath_in)
            content, ext = self.render(relpath_in)
            relpath_out = '%s%s' % (fn, ext)
            path_in = os.path.join(self.views_dir, relpath_in)
            path_out = utils.make_dirs(self.build_dir, relpath_out)

            if not content:
                return shutil.copy2(path_in, path_out)
            if ext == '.html':
                return views.append([relpath_in, path_out, content])
            if ext != old_ext:
                processed.append([relpath_in, relpath_out])
            utils.make_file(path_out, content)
            return
        
        utils.walk_dir(self.views_dir, callback, IGNORE)
        rx_processed = utils.get_processed_regex(processed)
        views_list = []

        for relpath_in, path_out, content in views:
            content = utils.absolute_to_relative(content, relpath_in)
            content = utils.replace_processed_names(content, rx_processed)
            utils.make_file(path_out, content)
            views_list.append(relpath_in.decode('utf8'))
        
        self.build_views_list(views_list)
    
    def build_views_list(self, views):
        ignore = self.settings.views_list_ignore
        ignore.append(VIEWS_INDEX)

        views = [
            (
                v,
                ' / '.join(v.split('/')),
                utils.get_file_mdate(os.path.join(self.views_dir, v)),
            ) \
            for v in views \
                if v not in ignore and not v.startswith(IGNORE)
            ]
        content, mimetype = self.render(VIEWS_INDEX, views=views)
        final_path = utils.make_dirs(self.build_dir, VIEWS_INDEX)
        utils.make_file(final_path, content)
    
    def not_found(self):
        content, ext = self.render('notfound.html')
        resp = Response(content)
        resp.status_code = 404
        return resp

